@@ -4,6 +4,7 @@
 A Qt-based image viewer which supports zooming and scrolling.
 """
 
+import os
 from enum import Enum
 from PyQt5.QtWidgets import QMainWindow, QApplication, QWidget, QScrollArea,\
     QHBoxLayout, QVBoxLayout, QDialog
@@ -43,6 +44,40 @@ class ImageLabel(QWidget):
             (event.rect().height() - scaled.height()) // 2)
         painter.drawPixmap(pos, scaled)
 
+def isDroppableMimeType(mime_data):
+    """
+    Returns True if the given mime_data belongs to a file which can
+    be dropped onto the ImageViewer/Canvas, i.e. it must be either a file
+    or a URI of a local file (as we won't support downloading files for now).
+    """
+    if mime_data.hasUrls():
+        is_local_file = [os.path.exists(url.toLocalFile()) for url in mime_data.urls()]
+        if any(is_local_file):
+            return True
+    if mime_data.hasText():
+        if mime_data.text().startswith('file://'):
+            return True
+    return False
+
+
+def getDroppableFilename(mime_data):
+    """
+    Returns the filename of a file dropped into the canvas (if it was
+    accepted via @see isDroppableMimeType).
+    """
+    if mime_data.hasUrls():
+        # Return the first locally existing file
+        for url in mime_data.urls():
+            fpath = url.toLocalFile()
+            if os.path.exists(fpath):
+                return fpath.strip()
+    if mime_data.hasText():
+        txt = mime_data.text()
+        if txt.startswith('file://'):
+            return txt[7:].strip()
+    raise ValueError('Unsupported QMimeData for dropped file!')
+
+
 
 class ImageCanvas(QWidget):
     """Widget to display a zoomable/scrollable image."""
@@ -56,6 +91,8 @@ class ImageCanvas(QWidget):
     rectSelected = pyqtSignal(tuple)
     # Scaling factor of displayed image changed
     imgScaleChanged = pyqtSignal(float)
+    # File has been dropped onto canvas
+    filenameDropped = pyqtSignal(str)
 
     def __init__(
             self, parent=None, rect_selectable=False,
@@ -74,6 +111,7 @@ class ImageCanvas(QWidget):
         self._is_dragging = False
         self._prev_drag_pos = None  # Parent widget position, i.e. usually the position within the ImageViewer (scroll area )
         self.setMouseTracking(True)
+        self.setAcceptDrops(True)
 
     def setScale(self, scale):
         prev_scale = self._scale
@@ -157,6 +195,17 @@ class ImageCanvas(QWidget):
             self._is_dragging = False
             QApplication.restoreOverrideCursor()
 
+    def dragEnterEvent(self, event):
+        if isDroppableMimeType(event.mimeData()):
+            event.accept()
+        else:
+            event.ignore()
+
+    def dropEvent(self, event):
+        if isDroppableMimeType(event.mimeData()):
+            fpath = getDroppableFilename(event.mimeData())
+            self.filenameDropped.emit(fpath)
+
     def paintEvent(self, event):
         if not self._pixmap:
             return super(ImageCanvas, self).paintEvent(event)
@@ -300,6 +349,8 @@ class ImageViewer(QScrollArea):
     imgScaleChanged = pyqtSignal(float)
     # The view changed due to the user scrolling or zooming
     viewChanged = pyqtSignal()
+    # File has been dropped onto canvas
+    filenameDropped = pyqtSignal(str)
 
     def __init__(self, parent=None, viewer_type=ImageViewerType.VIEW_ONLY, **kwargs):
         super(ImageViewer, self).__init__(parent)
@@ -344,6 +395,7 @@ class ImageViewer(QScrollArea):
         self._canvas.mouseMoved.connect(self.mouseMoved)
         self._canvas.imgScaleChanged.connect(self.imgScaleChanged)
         self._canvas.imgScaleChanged.connect(lambda _: self.viewChanged.emit())
+        self._canvas.filenameDropped.connect(self.filenameDropped)
 
         self.setWidget(self._canvas)
         self.setWidgetResizable(True)
