@@ -1,50 +1,65 @@
 #!/usr/bin/env python
 # coding=utf-8
 """Inspect matrix/image data"""
-# TODO Usability improvement:
-# Status bar for each InspectionWidget where the pixel value is shown (i.e.
-# you can inspect a pixel value for multiple images at once).
-# However, this is only useful if the displayed images have the same size and
-# are "linked".
-
-# TODO Usability improvement:
-# Let the user explicitly enable/disable linking of the viewers. Currently,
-# viewers will always be "linked" if all images have the same width/height.
-
-# TODO Usability improvement:
-# Implement a range slider widget which allows the user to adjust the
-# pseudocolor value limits on-the-fly.
-
-# TODO Usability improvement:
-# When showing multiple images, add an option to use the same visualization
-# data range across all images.
-
-# TODO Usability improvement:
-# Incrementally in-/decreasing the zoom factor worked "good enough" so far
-# however, "fast zooming" seems a bit "too fast" sometimes. Especially "fast
-# zooming" out is really fast (decreases by 50% with each wheel tick).
-
-# TODO GUI issue:
-# Initial window resize won't scale to the exact specified size.
-# QApplication().instance().processEvents() doesn't help either. The widgets
-# (e.g. image canvas) will be resized "shortly" after initializing the QMainWindow
-# for a second time. Needs thorough investigation.
 
+# TODO Ideas and potential usability improvements:
+# * Load multiple files via drag & drop from file browser/external image viewer
+#   => requires layout reset; potential issue: variable number of inspection
+#      widgets (what's the expected behavior if there are already multple widgets
+#      open for inspection?)
+#
+# * Handle infinite and NaN values
+#   => Extend the data overview/summary label: "Contains special values: NaN, Inf" 
+#      (list only those that are actually present)
+#
+# * Add input boxes to range slider for manually typing the upper/lower limits
+#
+# * GUI issue:
+#   Initial window resize won't scale to the exact specified size.
+#   QApplication().instance().processEvents() doesn't help either. The widgets
+#   (e.g. image canvas) will be resized "shortly" after initializing the QMainWindow
+#   for a second time. Needs thorough investigation.
+#
+# * Usability improvement:
+#   Incrementally in-/decreasing the zoom factor worked "good enough" so far
+#   however, "fast zooming" seems a bit "too fast" sometimes. Especially "fast
+#   zooming" out is really fast (decreases by 50% with each wheel tick).
+#
+# * Usability improvement:
+#   Status bar for each InspectionWidget where the pixel value is shown (i.e.
+#   you can inspect a pixel value for multiple images at once).
+#   However, this is only useful if the displayed images have the same size and
+#   are "linked".
+#
+# * Usability improvement:
+#   Let the user explicitly enable/disable linking of the viewers. Currently,
+#   viewers will always be "linked" if all images have the same width/height.
+#
+# * Usability improvement:
+#   Implement a range slider widget which allows the user to adjust the
+#   pseudocolor value limits on-the-fly.
+#
+# * Usability improvement:
+#   When showing multiple images, add an option to use the same visualization
+#   data range across all images.
+#
 # * Potential (usability) issue:
-# Consider this scenario: Initially, the user may have shown two same-sized
-# images (viewer axes have been linked). Now, a differently sized image has
-# been opened. Thus, the viewers will still/again be linked which may lead
-# to minor zooming/scrolling issues.
-# Currently, I prefer not to deal with such unexpected user behavior, as
-# this increases the code complexity unnecessarily.
+#   Consider this scenario: Initially, the user may have shown two same-sized
+#   images (viewer axes have been linked). Now, a differently sized image has
+#   been opened. Thus, the viewers will still/again be linked which may lead
+#   to minor zooming/scrolling issues.
+#   Currently, I prefer not to deal with such unexpected user behavior, as
+#   this increases the code complexity unnecessarily.
 
 import numpy as np
+import os
 from enum import Enum
 from PyQt5.QtWidgets import QMainWindow, QApplication, QWidget, \
     QHBoxLayout, QVBoxLayout, QGridLayout, QLabel, QFrame, QToolTip, \
     QShortcut, QMessageBox, QScrollArea, QSizePolicy
 from PyQt5.QtCore import Qt, QSize, QPoint, pyqtSignal, pyqtSlot
 from PyQt5.QtGui import QCursor, QFont, QKeySequence, QResizeEvent
+from PIL import UnidentifiedImageError
 
 from vito import imutils
 from vito import colormaps
@@ -129,6 +144,31 @@ class DataType(Enum):
                 return DataType.MULTICHANNEL
         else:
             raise ValueError('Input data with ndim > 3 (i.e. %d) is not supported!' % npdata.ndim)
+    
+    @staticmethod
+    def fromFilename(filename):
+        """
+        Returns the best guess on the proper data type given the filename.
+        In particular, we check the extension and return:
+        * *.flo files: DataType.FLOW
+        * *.npy files: DataType.MULTICHANNEL
+        * Image files will be opened with default settings and are either DataType.MONOCHROME or .COLOR
+        Raises a ValueError if the file extension is not a supported image extension.
+        """
+        _, ext = os.path.splitext(filename.lower())
+        if ext == '.flo':
+            return DataType.FLOW
+        elif ext == '.npy':
+            return DataType.MULTICHANNEL
+        else:
+            try:
+                data = imutils.imread(filename)
+                if data.ndim < 3 or data.shape[2] == 1:
+                    return DataType.MONOCHROME
+                return DataType.COLOR
+            except UnidentifiedImageError:
+                raise ValueError('Filename is not a loadable image.')
+        return DataType.COLOR
 
     @staticmethod
     def pilModeFor(data_type, data=None):
@@ -306,9 +346,12 @@ class InspectionWidget(QWidget):
 
     def getPixelValue(self, px_x, px_y):
         """Retrieves the image data at location (px_x, px_y)."""
+        if self._data is None:
+            return None
         x = int(px_x)
         y = int(px_y)
-        if x < 0 or x >= self._data.shape[1] or y < 0 or y >= self._data.shape[0]:
+        width = self._data.shape[1] if self._data.ndim > 1 else 1
+        if x < 0 or x >= width or y < 0 or y >= self._data.shape[0]:
             return None
         query = dict()
         query['pos'] = '({:d}, {:d})'.format(x, y)
@@ -316,7 +359,7 @@ class InspectionWidget(QWidget):
         # Representation of raw data
         query['currlayer'] = None
         if self._is_single_channel:
-            value = self._data[y, x]
+            value = self._data[y] if self._data.ndim == 1 else self._data[y, x]
             if self._data_type == DataType.CATEGORICAL \
                     and self._categorical_labels is not None \
                     and value in self._categorical_labels:
@@ -437,6 +480,29 @@ class InspectionWidget(QWidget):
             msg.setWindowTitle('Error')
             msg.exec()
 
+    @pyqtSlot(str)
+    def __openDroppedFilename(self, filename):
+        if filename is None:
+            return
+        # 1) Guess datatype
+        try:
+            dtype = DataType.fromFilename(filename)
+        except ValueError:
+            msg = QMessageBox()
+            msg.setIcon(QMessageBox.Critical)
+            msg.setText('{:s} is not a supported image file.'.format(filename))
+            msg.setWindowTitle('Error')
+            msg.exec()
+            return
+        # 2) Initialize open dialog (pre-set filename and datatype)
+        self._open_file_dialog = inspection_widgets.OpenInspectionFileDialog(
+            data_type=dtype,
+            thumbnail=None if self._data is None else self._img_viewer.imagePixmap(),
+            filename_suggestion=filename,
+            parent=self)
+        self._open_file_dialog.finished.connect(self.__onOpenFinished)
+        self._open_file_dialog.open()
+
     @pyqtSlot()
     def __onOpenFinished(self):
         res = self._open_file_dialog.getSelection()
@@ -635,6 +701,7 @@ class InspectionWidget(QWidget):
         self._img_viewer.mouseMoved.connect(lambda px: self.showTooltipRequest.emit(self._inspector_id, px))
         self._img_viewer.viewChanged.connect(lambda: self.showTooltipRequest.emit(self._inspector_id, None))
         self._img_viewer.imgScaleChanged.connect(lambda s: self.imgScaleChanged.emit(self._inspector_id, s))
+        self._img_viewer.filenameDropped.connect(self.__openDroppedFilename)
         self._img_viewer.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         img_layout.addWidget(self._img_viewer)
 
@@ -870,7 +937,7 @@ class Inspector(QMainWindow):
                 layout.addWidget(insp,
                     idx // max_num_widgets_per_row, idx % max_num_widgets_per_row)
                 if data[idx].shape[0] != data[0].shape[0] \
-                        or data[idx].shape[1] != data[0].shape[1]:
+                    or (data[idx].ndim > 1 and data[0].ndim > 1 and data[idx].shape[1] != data[0].shape[1]):
                     matching_input_shape = False
             # Turn off scale display if images have different resolution
             self._zoom_widget.showScaleLabel(matching_input_shape)
